@@ -174,10 +174,12 @@ add_vevent (ECalBackendExchange *cbex,
 		icalproperty_free (prop);
 	}
 
+	e_cal_backend_exchange_cache_lock (cbex);
 	/* Now add to the cache */
 	status = e_cal_backend_exchange_add_object (cbex, href, lastmod, icalcomp);
+	e_cal_backend_exchange_cache_unlock (cbex);
 	
-	return (status == GNOME_Evolution_Calendar_Success);
+	return status;
 }
 
 /* Add the event to the cache, Notify the backend if it is sucessfully added */
@@ -233,7 +235,9 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 		status = add_vevent (cbex, href, lastmod, icalcomp);
 
 		if (status) {
-			e_cal_backend_notify_object_created (backend, icalcomponent_as_ical_string (icalcomp));
+			char *object = g_strdup (icalcomponent_as_ical_string (icalcomp));
+			e_cal_backend_notify_object_created (backend, object);
+			g_free (object);
 		}
 
 		icalcomponent_free (icalcomp);
@@ -266,7 +270,9 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 			status = add_vevent (cbex, href, lastmod, new_comp);
 			
 			if (status) {
-				e_cal_backend_notify_object_created (backend, icalcomponent_as_ical_string (new_comp));
+				char *object = g_strdup (icalcomponent_as_ical_string (new_comp));
+				e_cal_backend_notify_object_created (backend, object);
+				g_free (object);
 			}
 
 			icalcomponent_free (new_comp);
@@ -339,6 +345,7 @@ get_changed_events (ECalBackendExchange *cbex)
 					   cbex->private_item_restriction,
 					   NULL);
 	}
+	e_cal_backend_exchange_cache_lock (cbex);
 	if (since) {
 		rn = e2k_restriction_andv (
 			rn,
@@ -347,6 +354,7 @@ get_changed_events (ECalBackendExchange *cbex)
 			NULL);
 	} else
 		e_cal_backend_exchange_cache_sync_start (cbex);
+	e_cal_backend_exchange_cache_unlock (cbex);
 
 	iter = e_folder_exchange_search_start (cbex->folder, NULL,
 					       event_properties,
@@ -385,6 +393,7 @@ get_changed_events (ECalBackendExchange *cbex)
 				status_tracking = EX_READ_AND_DELIVERED;
 		}
 
+		e_cal_backend_exchange_cache_lock (cbex);
 		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime, result->href)) {
 			g_ptr_array_add (hrefs, g_strdup (result->href));
 			g_hash_table_insert (modtimes, g_strdup (result->href),
@@ -393,6 +402,7 @@ get_changed_events (ECalBackendExchange *cbex)
 				g_hash_table_insert (attachments, g_strdup (result->href),
 						g_strdup (uid));
 		}
+		e_cal_backend_exchange_cache_unlock (cbex);
 	}
 	status = e2k_result_iter_free (iter);
 
@@ -404,8 +414,10 @@ get_changed_events (ECalBackendExchange *cbex)
 		return status;
 	}
 
+	e_cal_backend_exchange_cache_lock (cbex);
 	if (!since)
 		e_cal_backend_exchange_cache_sync_end (cbex);
+	e_cal_backend_exchange_cache_unlock (cbex);
 
 	if (!hrefs->len) {
 		g_ptr_array_free (hrefs, TRUE);
@@ -465,12 +477,14 @@ get_changed_events (ECalBackendExchange *cbex)
 	if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
 		g_ptr_array_free (hrefs, TRUE);
 		g_hash_table_destroy (modtimes);
+		g_hash_table_destroy (attachments);
 		g_mutex_unlock (cbexc->priv->mutex);
 		return status;
 	}
 	if (!hrefs->len) {
 		g_ptr_array_free (hrefs, TRUE);
 		g_hash_table_destroy (modtimes);
+		g_hash_table_destroy (attachments);
 		cbexc->priv->is_loaded = TRUE;
 		g_mutex_unlock (cbexc->priv->mutex);
 		return SOUP_STATUS_OK;
@@ -650,10 +664,10 @@ find_attendee_prop (icalcomponent *ical_comp, const char *address)
 
 		attendee = icalvalue_get_string (value);
 
-		if (!g_strncasecmp (attendee, "mailto:", 7))
+		if (!g_ascii_strncasecmp (attendee, "mailto:", 7))
 			attendee += 7;
 		text = g_strstrip (g_strdup (attendee));
-		if (!g_strcasecmp (address, text)) {
+		if (!g_ascii_strcasecmp (address, text)) {
 			g_free (text);
 			return prop;
 		}
@@ -673,6 +687,7 @@ check_owner_partstatus_for_declined (ECalBackendSync *backend,
 	
 	email = e_cal_backend_exchange_get_owner_email (backend);
 	icalprop = find_attendee_prop (icalcomp, email);
+
 	g_free (email);
 	if (!icalprop)
 		return FALSE;
@@ -691,6 +706,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 {
 	/* FIXME : Return some value in uid */
 	ECalBackendExchangeCalendar *cbexc;
+	ECalBackendExchange *cbex;
 	icalcomponent *icalcomp, *real_icalcomp;
 	icalcomponent_kind kind;
 	icalproperty *icalprop;
@@ -714,6 +730,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	d(printf ("ecbexc_create_object(%p, %p, %s, %s)", backend, cal, *calobj ? *calobj : NULL, *uid ? *uid : NULL));
 
 	cbexc =	E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
+	cbex = E_CAL_BACKEND_EXCHANGE (backend);
 	
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_CALENDAR (cbexc), GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
@@ -876,8 +893,10 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 
+	e_cal_backend_exchange_cache_lock (cbex);
 	/*add object*/
 	e_cal_backend_exchange_add_object (E_CAL_BACKEND_EXCHANGE (cbexc), location, lastmod, icalcomp);
+	e_cal_backend_exchange_cache_unlock (cbex);
 	*uid = g_strdup (temp_comp_uid);	
 		
 	g_object_unref (comp);
@@ -1006,9 +1025,14 @@ modify_object (ECalBackendSync *backend, EDataCal *cal,
 	       const char *calobj, CalObjModType mod,
 	       char **old_object, char **new_object)
 {
+	ECalBackendSyncStatus status;
+	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
+
 	d(printf ("ecbexc_modify_object(%p, %p, %d, %s)", backend, cal, mod, *old_object ? *old_object : NULL));
 
-	return modify_object_with_href (backend, cal, calobj, mod, old_object, new_object, NULL, NULL);
+	status = modify_object_with_href (backend, cal, calobj, mod, old_object, new_object, NULL, NULL);
+
+	return status;
 }
 
 static ECalBackendSyncStatus
@@ -1018,6 +1042,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 {
 	ECalBackendExchangeCalendar *cbexc;
 	ECalBackendExchangeComponent *ecomp;
+	ECalBackendExchange *cbex;
 	icalcomponent *icalcomp, *real_icalcomp, *updated_icalcomp;
 	ECalComponent *real_ecomp, *cached_ecomp = NULL, *updated_ecomp;
 	const char *comp_uid;
@@ -1040,8 +1065,11 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	GList *l = NULL;
 	struct icaltimetype inst_rid;
 	gboolean remove = FALSE;
+	ECalBackendSyncStatus status = GNOME_Evolution_Calendar_Success;
+	GList *to_remove = NULL;
 
 	cbexc =	E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
+	cbex = E_CAL_BACKEND_EXCHANGE (backend);
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_CALENDAR (cbexc), GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
@@ -1071,11 +1099,12 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	else  
 		key_rid = icaltime_from_string (rid_to_remove);
 	
-	
-	ecomp = get_exchange_comp (E_CAL_BACKEND_EXCHANGE (cbexc), comp_uid);
+	e_cal_backend_exchange_cache_lock (cbex);
+	ecomp = get_exchange_comp (cbex, comp_uid);
 	
 	if (!ecomp){
 		icalcomponent_free (icalcomp);
+		e_cal_backend_exchange_cache_unlock (cbex);
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	}
 	
@@ -1105,16 +1134,18 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 			end = icalcomponent_get_dtend (icalcomp);
 	
 			if (icaltime_compare (start, key_rid) != 0) {
-					icaltimetype end = icalcomponent_get_dtend (icalcomp);
-					
 					m_start.hour = start.hour;
 					m_start.minute = start.minute;
 					m_start.second = start.second;
 						
+			}
+
+			if (!((m_end.hour == end.hour) && (m_end.minute == end.minute) && (m_end.second == end.second))) {
 					m_end.hour = end.hour;
 					m_end.minute = end.minute;
 					m_end.second = end.second;
-			} 
+			}
+
 			icalcomponent_set_dtstart (icalcomp, m_start);
 			icalcomponent_set_dtend (icalcomp, m_end);
 			
@@ -1144,6 +1175,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	g_free (updated_ecomp_str);
 	if (!updated_icalcomp) {
 		g_object_unref (updated_ecomp);
+		e_cal_backend_exchange_cache_unlock (cbex);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 
@@ -1164,6 +1196,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	if (!e_cal_component_set_icalcomponent (real_ecomp, updated_icalcomp)) {
 		g_object_unref (real_ecomp);
 		g_object_unref (updated_ecomp);
+		e_cal_backend_exchange_cache_unlock (cbex);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 
@@ -1241,8 +1274,13 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		e_cal_component_free_recur_list (rrule_list);
 	}
 
+	if (mod == CALOBJ_MOD_ALL && ecomp->icomp) {
+		cached_ecomp = e_cal_component_new ();
+		e_cal_component_set_icalcomponent (cached_ecomp, icalcomponent_new_clone (ecomp->icomp));
+		if (e_cal_component_has_recurrences (real_ecomp))
+			e_cal_component_set_recurid (real_ecomp, NULL);
+	}
 
-	
 	/* add the timezones information and the component itself
 	   to the VCALENDAR object */
 	e_cal_component_commit_sequence (real_ecomp);
@@ -1252,6 +1290,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		g_object_unref (updated_ecomp);	
 		icalcomponent_free (cbdata->vcal_comp);
 		g_free (cbdata);
+		e_cal_backend_exchange_cache_unlock (cbex);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 	real_icalcomp = icalparser_parse_string (real_comp_str);
@@ -1269,7 +1308,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	if (ecomp->icomp && mod == CALOBJ_MOD_THIS) {
 		icalcomponent_add_component (cbdata->vcal_comp, icalcomponent_new_clone (ecomp->icomp));
 	}
-
+	
 	for (l = ecomp->instances; l != NULL; l = l->next) {
 		icalcomponent *icomp = l->data;
 
@@ -1278,20 +1317,30 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 			cached_ecomp = e_cal_component_new ();
 
 			e_cal_component_set_icalcomponent (cached_ecomp,
-					icalcomponent_new_clone (l->data));
-			if (remove) {
-				icalcomponent_free (l->data);
-				l = g_list_remove_link (ecomp->instances, l);
-				if (!l)
-					break;
-			}
-
+					icalcomponent_new_clone (icomp));
+			if (remove) 
+				to_remove = l;
 			continue;
 		} else {
-			icalcomponent_add_component (cbdata->vcal_comp, icalcomponent_new_clone (icomp));
+			ECalComponent *comp = e_cal_component_new ();
+			e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (icomp));
+
+			update_x_properties (E_CAL_BACKEND_EXCHANGE (cbexc), comp);
+
+			icalcomponent_add_component (cbdata->vcal_comp, 
+					icalcomponent_new_clone (e_cal_component_get_icalcomponent (comp)));
+
+			g_object_unref (comp);
 		}
 	}
 
+	if (to_remove) {
+		icalcomponent_free (to_remove->data);
+		ecomp->instances = g_list_remove_link (ecomp->instances, to_remove);
+	}
+
+	e_cal_backend_exchange_cache_unlock (cbex);
+
 	if (!cached_ecomp && remove)
 		*new_object = g_strdup (icalcomponent_as_ical_string (icalcomp));
 
@@ -1366,9 +1415,13 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	if ((E2K_HTTP_STATUS_IS_SUCCESSFUL (http_status)) && send_options)
 		http_status = e2k_context_proppatch (ctx, NULL, new_href, props, FALSE, NULL);
 
-	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (http_status))
+	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (http_status)) {
+		e_cal_backend_exchange_cache_lock (cbex);
 		e_cal_backend_exchange_modify_object (E_CAL_BACKEND_EXCHANGE (cbexc), 
 							e_cal_component_get_icalcomponent (real_ecomp), mod, remove);
+		e_cal_backend_exchange_cache_unlock (cbex);
+	} else 
+	 	status = GNOME_Evolution_Calendar_OtherError;		
 	
 	if (!remove)
 		*new_object = e_cal_component_get_as_string (real_ecomp);
@@ -1384,7 +1437,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	g_free (cbdata);
 	e2k_properties_free (props);
 
-	return GNOME_Evolution_Calendar_Success;
+	return status;
 }
  
 static ECalBackendSyncStatus
@@ -1393,6 +1446,7 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 	       char **old_object, char **object)
 {
 	ECalBackendExchangeCalendar *cbexc;
+	ECalBackendExchange *cbex;
 	ECalBackendExchangeComponent *ecomp;
 	E2kHTTPStatus status;
 	E2kContext *ctx;
@@ -1406,6 +1460,7 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 	   handled. */
 	
 	cbexc = E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
+	cbex = E_CAL_BACKEND_EXCHANGE (backend);
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_CALENDAR (cbexc), GNOME_Evolution_Calendar_InvalidObject);
 
@@ -1414,10 +1469,13 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
                 return GNOME_Evolution_Calendar_InvalidObject;
         }
 
+	e_cal_backend_exchange_cache_lock (cbex);
 	ecomp = get_exchange_comp (E_CAL_BACKEND_EXCHANGE (cbexc), uid);
 	
-	if (!ecomp)
+	if (!ecomp) {
+		e_cal_backend_exchange_cache_unlock (cbex);
 		return GNOME_Evolution_Calendar_ObjectNotFound;
+	}
 		
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->icomp));
@@ -1434,6 +1492,8 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 		time_rid = icaltime_from_string (rid);
 		e_cal_util_remove_instances (ecomp->icomp, time_rid, mod);
 		calobj  = (char *) icalcomponent_as_ical_string (ecomp->icomp);
+		
+		e_cal_backend_exchange_cache_unlock (cbex);
 		ebs_status = modify_object_with_href (backend, cal, calobj, mod, &obj, &new_object, NULL, rid);
 		g_object_unref (comp);
 		if (ebs_status != GNOME_Evolution_Calendar_Success)
@@ -1446,16 +1506,21 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 		g_free (new_object);
 
 		return ebs_status;
-	}
+	} else 
+		e_cal_backend_exchange_cache_unlock (cbex);
+
 	g_object_unref (comp);
 
 	ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);	
 	
 	status = e2k_context_delete (ctx, NULL, ecomp->href);
 	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
+		e_cal_backend_exchange_cache_lock (cbex);
 		if (e_cal_backend_exchange_remove_object (E_CAL_BACKEND_EXCHANGE (cbexc), uid)) {
+			e_cal_backend_exchange_cache_unlock (cbex);
 			return GNOME_Evolution_Calendar_Success;
 		}
+		e_cal_backend_exchange_cache_unlock (cbex);
 	}
 	*object = NULL;
 
@@ -1469,6 +1534,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 {
 	d(printf ("ecbexc_modify_object(%p, %p, %s)", backend, cal, calobj ? calobj : NULL));
 	ECalBackendExchangeCalendar *cbexc;
+	ECalBackendExchange *cbex;
 	ECalBackendExchangeComponent *ecomp;
 	ECalComponent *comp = NULL;
 	GList *comps, *l;
@@ -1478,6 +1544,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 	ECalBackendSyncStatus status = GNOME_Evolution_Calendar_Success;	
 
 	cbexc =	E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
+	cbex =	E_CAL_BACKEND_EXCHANGE (backend);
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_CALENDAR (cbexc), GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
@@ -1518,29 +1585,29 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 		case ICAL_METHOD_PUBLISH:
 		case ICAL_METHOD_REQUEST:
 		case ICAL_METHOD_REPLY:
+			e_cal_backend_exchange_cache_lock (cbex);
 			if ((ecomp = get_exchange_comp (E_CAL_BACKEND_EXCHANGE (cbexc), uid)) != NULL ) {
 				char *old_object = NULL;
 
 				d(printf ("uid : %s : found in the cache\n", uid));
 
+				e_cal_backend_exchange_cache_unlock (cbex);
 				if (check_owner_partstatus_for_declined (backend, subcomp)) {
+					ECalComponentId *id = NULL;
 					status = remove_object (backend, cal, uid, NULL, 
 								CALOBJ_MOD_ALL, &old_object,
 								NULL);
 					if (status != GNOME_Evolution_Calendar_Success)
 						goto error;
-					e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid,
+					id = e_cal_component_get_id (comp);
+					e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), id,
 									     old_object, NULL);
+					e_cal_component_free_id (id);
 				} else {
 					struct icaltimetype time_rid;
 					char *new_object = NULL;
 					CalObjModType mod = CALOBJ_MOD_ALL;
 					
-					if (rid) {
-						time_rid = icaltime_from_string (rid);
-						e_cal_util_remove_instances (ecomp->icomp, time_rid, CALOBJ_MOD_THIS);
-					}
-
 					if (e_cal_util_component_is_instance (subcomp))
 						mod = CALOBJ_MOD_THIS;
 
@@ -1561,17 +1628,24 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 				g_free (old_object);
 			} else if (!check_owner_partstatus_for_declined (backend, subcomp)) {
 				d(printf ("object : %s .. not found in the cache\n", uid));
-				char *returned_uid, *old;
+				char *returned_uid, *old, *object;
 				icalobj = (char *) icalcomponent_as_ical_string (subcomp);
 				d(printf ("Create a new object : %s\n", icalobj));
+				
+				e_cal_backend_exchange_cache_unlock (cbex);
 				status = create_object (backend, cal, &icalobj, &returned_uid);
 				if (status != GNOME_Evolution_Calendar_Success)
 					goto error;
 
+				object = g_strdup (icalobj);
 				e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), icalobj);
+				g_free (object);
 				d(printf ("Notify that the new object is created : %s\n", icalobj));
-			} else
+			} else {
+				e_cal_backend_exchange_cache_unlock (cbex);
 				status = GNOME_Evolution_Calendar_Success;
+			}
+
 			break;
 		case ICAL_METHOD_ADD:
 			/* FIXME This should be doable once all the recurid stuff is done ??*/
@@ -1583,8 +1657,12 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 				status = remove_object (backend, cal, uid, rid, CALOBJ_MOD_THIS, &icalobj, &object);
 			else
 				status = remove_object (backend, cal, uid, NULL, CALOBJ_MOD_ALL, &icalobj, &object);
-			if (status == GNOME_Evolution_Calendar_Success) 
-				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid, icalobj, NULL);
+			if (status == GNOME_Evolution_Calendar_Success) {
+				ECalComponentId *id = e_cal_component_get_id (comp);
+				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), id, icalobj, NULL);
+				e_cal_component_free_id (id);
+				
+			}
 			if (object) {
 				g_free (object);
 				object = NULL;
@@ -1717,7 +1795,10 @@ book_resource (ECalBackendExchange *cbex,
 
 	e_cal_component_get_uid (E_CAL_COMPONENT (comp), &uid);
 	href = g_strdup_printf ("%s/%s.EML", cal_uri, uid);
+	
+	e_cal_backend_exchange_cache_lock (cbex);
 	ecomp = get_exchange_comp (E_CAL_BACKEND_EXCHANGE (cbex), uid);
+	e_cal_backend_exchange_cache_unlock (cbex);
 												 
 	if (method == ICAL_METHOD_CANCEL) {
 		char *object = NULL;
@@ -2063,14 +2144,16 @@ discard_alarm (ECalBackendSync *backend, EDataCal *cal,
 
 	cbex = E_CAL_BACKEND_EXCHANGE (backend);
 
+	e_cal_backend_exchange_cache_lock (cbex);
 	ecbexcomp = get_exchange_comp (cbex, uid);
 
-	if (!ecbexcomp) 
+	if (!ecbexcomp) {
+		e_cal_backend_exchange_cache_unlock (cbex);
 		return GNOME_Evolution_Calendar_ObjectNotFound;
+	}
 
 	ecomp = e_cal_component_new ();
-	e_cal_component_set_icalcomponent (ecomp, icalcomponent_new_clone (ecbexcomp->icomp));
-	if (!e_cal_component_has_recurrences (ecomp))
+	if (e_cal_component_set_icalcomponent (ecomp, icalcomponent_new_clone (ecbexcomp->icomp)) && !e_cal_component_has_recurrences (ecomp))
 	{
 		e_cal_component_remove_alarm (ecomp, auid);
 		ecomp_str = e_cal_component_get_as_string (ecomp);
@@ -2084,6 +2167,7 @@ discard_alarm (ECalBackendSync *backend, EDataCal *cal,
 	}
 	g_object_unref (ecomp);
 
+	e_cal_backend_exchange_cache_unlock (cbex);
 	return result;
 }
 
